/*
 * Copyright (c) 2002-2003, Network Appliance, Inc. All rights reserved.
 *
 * This Software is licensed under either one of the following two licenses:
 *
 * 1) under the terms of the "Common Public License 1.0" a copy of which is
 *    in the file LICENSE.txt in the root directory. The license is also
 *    available from the Open Source Initiative, see
 *    http://www.opensource.org/licenses/cpl.php.
 * OR
 *
 * 2) under the terms of the "The BSD License" a copy of which is in the file
 *    LICENSE2.txt in the root directory. The license is also available from
 *    the Open Source Initiative, see
 *    http://www.opensource.org/licenses/bsd-license.php.
 *
 * Licensee has the right to choose either one of the above two licenses.
 *
 * Redistributions of source code must retain both the above copyright
 * notice and either one of the license notices.
 *
 * Redistributions in binary form must reproduce both the above copyright
 * notice, either one of the license notices in the documentation
 * and/or other materials provided with the distribution.
 */

/**********************************************************************
 *
 * HEADER: udat_redirection.h
 *
 * PURPOSE: User DAT macro definitions
 *
 * Description: Header file for "uDAPL: User Direct Access Programming
 *		Library, Version: 1.1"
 *
 * Mapping rules:
 *      All global symbols are prepended with "DAT_" or "dat_"
 *      All DAT objects have an 'api' tag which, such as 'ep' or 'lmr'
 *      The method table is in the provider definition structure.
 *
 *
 *
 **********************************************************************/

#ifndef _UDAT_REDIRECTION_H_
#define _UDAT_REDIRECTION_H_

#define dat_lmr_create(ia, mtype, reg_desc, len, pz, priv, \
                       lmr, lmr_context, rmr_context, reg_len, reg_addr) \
         (*DAT_HANDLE_TO_PROVIDER(ia)->lmr_create_func)(\
                (ia), \
                (mtype), \
                (reg_desc), \
                (len), \
                (pz), \
                (priv), \
                (lmr), \
                (lmr_context), \
                (rmr_context), \
                (reg_len), \
                (reg_addr))

#define dat_evd_create(ia, qlen, cno, flags, handle) \
        (*DAT_HANDLE_TO_PROVIDER(ia)->evd_create_func)(\
                (ia), \
                (qlen), \
                (cno), \
                (flags), \
                (handle))

#define dat_evd_enable(evd) \
        (*DAT_HANDLE_TO_PROVIDER(evd)->evd_enable_func)(\
                (evd))

#define dat_evd_wait(evd, timeout, threshold, event, nmore) \
        (*DAT_HANDLE_TO_PROVIDER(evd)->evd_wait_func)(\
                (evd), \
                (timeout), \
                (threshold), \
                (event), \
                (nmore))

#define dat_evd_disable(evd) \
        (*DAT_HANDLE_TO_PROVIDER(evd)->evd_disable_func)(\
                (evd))

#define dat_evd_set_unwaitable(evd) \
        (*DAT_HANDLE_TO_PROVIDER(evd)->evd_set_unwaitable_func)(\
                (evd))

#define dat_evd_clear_unwaitable(evd) \
        (*DAT_HANDLE_TO_PROVIDER(evd)->evd_clear_unwaitable_func)(\
                (evd))

#define dat_evd_modify_cno(evd, cno) \
        (*DAT_HANDLE_TO_PROVIDER(evd)->evd_modify_cno_func)(\
                (evd), \
                (cno))

#define dat_cno_create(ia, proxy, cno) \
        (*DAT_HANDLE_TO_PROVIDER(ia)->cno_create_func)(\
                (ia), \
                (proxy), \
                (cno))

#define dat_cno_modify_agent(cno, proxy) \
        (*DAT_HANDLE_TO_PROVIDER(cno)->cno_modify_agent_func)(\
                (cno), \
                (proxy))

#define dat_cno_query(cno, mask, param) \
        (*DAT_HANDLE_TO_PROVIDER(cno)->cno_query_func)(\
                (cno), \
                (mask), \
                (param))

#define dat_cno_free(cno) \
        (*DAT_HANDLE_TO_PROVIDER(cno)->cno_free_func)(\
                (cno))

#define dat_cno_wait(cno, timeout, evd) \
        (*DAT_HANDLE_TO_PROVIDER(cno)->cno_wait_func)(\
                (cno), \
                (timeout), \
                (evd))

#define dat_cr_handoff(cr, qual) \
        (*DAT_HANDLE_TO_PROVIDER(cr)->cr_handoff_func)(\
                (cr), \
                (qual))

#include <dat/dat_redirection.h>

struct dat_provider
{
    const char *                        device_name;
    DAT_PVOID                           extension;

    DAT_IA_OPEN_FUNC                    ia_open_func;
    DAT_IA_QUERY_FUNC                   ia_query_func;
    DAT_IA_CLOSE_FUNC                   ia_close_func;

    DAT_SET_CONSUMER_CONTEXT_FUNC       set_consumer_context_func;
    DAT_GET_CONSUMER_CONTEXT_FUNC       get_consumer_context_func;
    DAT_GET_HANDLE_TYPE_FUNC            get_handle_type_func;

    DAT_CNO_CREATE_FUNC                 cno_create_func;	/* udat only */
    DAT_CNO_MODIFY_AGENT_FUNC           cno_modify_agent_func;	/* udat only */
    DAT_CNO_QUERY_FUNC                  cno_query_func;		/* udat only */
    DAT_CNO_FREE_FUNC                   cno_free_func;		/* udat only */
    DAT_CNO_WAIT_FUNC                   cno_wait_func;		/* udat only */

    DAT_CR_QUERY_FUNC                   cr_query_func;
    DAT_CR_ACCEPT_FUNC                  cr_accept_func;
    DAT_CR_REJECT_FUNC                  cr_reject_func;
    DAT_CR_HANDOFF_FUNC             cr_handoff_func;	    /* udat only */

    DAT_EVD_CREATE_FUNC                 evd_create_func;
    DAT_EVD_QUERY_FUNC                  evd_query_func;

    DAT_EVD_MODIFY_CNO_FUNC             evd_modify_cno_func;	/* udat only */
    DAT_EVD_ENABLE_FUNC                 evd_enable_func;	/* udat only */
    DAT_EVD_DISABLE_FUNC                evd_disable_func;	/* udat only */
    DAT_EVD_WAIT_FUNC                   evd_wait_func;		/* udat only */

    DAT_EVD_RESIZE_FUNC                 evd_resize_func;
    DAT_EVD_POST_SE_FUNC                evd_post_se_func;
    DAT_EVD_DEQUEUE_FUNC                evd_dequeue_func;
    DAT_EVD_FREE_FUNC                   evd_free_func;

    DAT_EP_CREATE_FUNC                  ep_create_func;
    DAT_EP_QUERY_FUNC                   ep_query_func;
    DAT_EP_MODIFY_FUNC                  ep_modify_func;
    DAT_EP_CONNECT_FUNC                 ep_connect_func;
    DAT_EP_DUP_CONNECT_FUNC             ep_dup_connect_func;
    DAT_EP_DISCONNECT_FUNC              ep_disconnect_func;
    DAT_EP_POST_SEND_FUNC               ep_post_send_func;
    DAT_EP_POST_RECV_FUNC               ep_post_recv_func;
    DAT_EP_POST_RDMA_READ_FUNC  ep_post_rdma_read_func;
    DAT_EP_POST_RDMA_WRITE_FUNC ep_post_rdma_write_func;
    DAT_EP_GET_STATUS_FUNC              ep_get_status_func;
    DAT_EP_FREE_FUNC                    ep_free_func;

    DAT_LMR_CREATE_FUNC                 lmr_create_func;
    DAT_LMR_QUERY_FUNC                  lmr_query_func;

    DAT_LMR_FREE_FUNC                   lmr_free_func;

    DAT_RMR_CREATE_FUNC                 rmr_create_func;
    DAT_RMR_QUERY_FUNC                  rmr_query_func;
    DAT_RMR_BIND_FUNC                   rmr_bind_func;
    DAT_RMR_FREE_FUNC                   rmr_free_func;

    DAT_PSP_CREATE_FUNC                 psp_create_func;
    DAT_PSP_QUERY_FUNC                  psp_query_func;
    DAT_PSP_FREE_FUNC                   psp_free_func;

    DAT_RSP_CREATE_FUNC                 rsp_create_func;
    DAT_RSP_QUERY_FUNC                  rsp_query_func;
    DAT_RSP_FREE_FUNC                   rsp_free_func;

    DAT_PZ_CREATE_FUNC                  pz_create_func;
    DAT_PZ_QUERY_FUNC                   pz_query_func;
    DAT_PZ_FREE_FUNC                    pz_free_func;

    /* udat-1.1 */
    DAT_PSP_CREATE_ANY_FUNC             psp_create_any_func;
    DAT_EP_RESET_FUNC                   ep_reset_func;
    DAT_EVD_SET_UNWAITABLE_FUNC         evd_set_unwaitable_func;
    DAT_EVD_CLEAR_UNWAITABLE_FUNC       evd_clear_unwaitable_func;

};

#endif /* _UDAT_REDIRECTION_H_ */


